function A = PoissonMatrix_Dirichlet(h,n)
% generate the matrix A for discrete Finite Difference
% Poisson equation
% input:
%   h: step size 
%   n: number of grid points per direction 
%      (numbering INCLUDES the boundary points)
% Dirichlet b.c. with boundary points included in the matrix
% inner points numbered separately like on the exercise sheet
%
% 02.05.2005 by vita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% A consists of 4 parts (see notes)
% [Ains Abd;
%  0    I];

% construct Ains
diag = 1/h/h*ones((n-2)^2,1);
x_diag = diag;
x_diag(n-2:n-2:end) = 0; % zeros at the end of the blocks
%keyboard
Ains = spdiags([diag [x_diag(1:end-1);NaN]  -4*diag [NaN; x_diag(1:end-1)] diag],...
               [-(n-2) -1 0 1 (n-2)],(n-2)^2,(n-2)^2);
%keyboard
% A on the boundary Abd
% initialise the memory
Abd = sparse((n-2)^2,4*n-4);
% go through all the boundary points

% boundary j=1,i=2...n-1
ctr = 1; % first corner ignored
j=1;
for i=2:n-1
  ctr = ctr+1;
  Abd(index(i,j+1,n),ctr) = 1/h/h;
end

ctr = ctr+1; % shift columns by one because of corner
% boundary i=n, j=2...n-1
i=n;
for j=2:n-1
   ctr = ctr+1;
   Abd(index(i-1,j,n),ctr) = 1/h/h;
end

ctr = ctr+1;
% boundary j=N, i=n-1...2
j=n;
for i=n-1:-1:2
  ctr = ctr+1;
  Abd(index(i,j-1,n),ctr) = 1/h/h;
end

ctr = ctr+1;
% boundary i=1, j=n-1...2
i=1;
for j=n-1:-1:2
  ctr = ctr+1;
  Abd(index(i+1,j,n),ctr) = 1/h/h;
end
  

A = [Ains Abd; sparse(4*n-4,(n-2)^2) speye(4*n-4)]; 



function I = index(i,j,n);
I = (j-2)*(n-2)+(i-1);



return

method = 'fast';
  if isequal(method,'slow')
  % slow version
    % assign inner points
    A = sparse(n,n);
    for i_ctr = 2:n-1
      for j_ctr = 2:n-1
        % find places for entries
        row_index = sub2ind([n,n],i_ctr,j_ctr);
	x_plus_col  = sub2ind([n,n],i_ctr+1,j_ctr);
	x_minus_col = sub2ind([n,n],i_ctr-1,j_ctr);
	y_plus_col  = sub2ind([n,n],i_ctr,j_ctr+1);
	y_minus_col = sub2ind([n,n],i_ctr,j_ctr-1);	
	  
	% fill the entries
	A(row_index,row_index)   = -4;
	A(row_index,x_plus_col)  = 1;
	A(row_index,x_minus_col) = 1;
	A(row_index,y_plus_col)  = 1;
	A(row_index,y_minus_col) = 1;	  	  	  
      end
    end
      
    % assign boundary points
    for i_ctr = 1:n
      % j=1
      row_index = sub2ind([n,n],i_ctr,1);
      A(row_index,row_index) = 1;
      % j=n
      row_index = sub2ind([n,n],i_ctr,n);
      A(row_index,row_index) = 1;
    end
    for j_ctr = 2:n-1 % (corners already assigned)
      % i=1
      row_index = sub2ind([n,n],1,j_ctr);
      A(row_index,row_index) = 1;
      % i=n
      row_index = sub2ind([n,n],n,j_ctr);
      A(row_index,row_index) = 1;	
    end	
      
  else
    % construct A with spdiags command!
    % for Matlab that's better!
      
    % boundary points will need a special treatment, 
    % so first sort them out
      
    bdry_x_ind = [[1:n]'; n*ones(n-2,1); [n:-1:1]'; ones(n-2,1)];
    bdry_y_ind = [ones(n,1); [2:n]'; n*ones(n-1,1); [n-1:-1:2]']; 
      
    bdry = sub2ind([n,n],bdry_x_ind,bdry_y_ind);
      
    % the main diagonal
    main_diag = ones(n*n,1);
    main_diag(bdry) = 0; % zeros on the diagonal
      
    % diagonal corresponding to x_{i+1}
    % dummy value at the beginning
    up_x_diag = [NaN; main_diag(1:n*n-1)];
      
    % diagonal corresponding to x_{i-1}
    % dummy value at the end
    low_x_diag = [main_diag(2:n*n);NaN];
      
    % diagonal corresponding to y_{j+1}
    % dummy values at the beginning
    up_y_diag = [NaN(n,1); main_diag(1:n*n-n)];      
            
    % diagonal corresponding to y_{j-1}
    % dummy values at the end
    low_y_diag = [main_diag(n+1:n*n); NaN(n,1)];      
           
    % put everything together
    A = spdiags([low_y_diag low_x_diag -4*main_diag up_x_diag up_y_diag],...
                [-n -1 0 1 n],n*n,n*n); 
     
    % add ones on the boundary
    bdry_diag = zeros(n*n,1);
    bdry_diag(bdry) = 1;
      
    A = A + spdiags(bdry_diag,0,n*n,n*n);
	
    A = sparse(A); % reoptimise the memory      	  
  end
    
