% script file BdryLayer
% demonstrate the boundary layers 
% equation
% Du + D^2u = f
% u(a) = uA, u(b) = uB;
% and also analyse the properties of 
% linear system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; clear all; %close all;

a = 0; b = 1;
lambda = -0.1; % diffusion
uA = 0; uB = 0;


ndom = [10:5:80];

%ndom = 20;

N = length(ndom);

% iteration parameters
maxit = 5000;
toler = 1e-1;

% store iteration count
iters.Richardson = zeros(1,N);  
iters.luinc0 = zeros(1,N);   time.luinc0 = zeros(1,N);
iters.luinc1 = zeros(1,N);   time.luinc1 = zeros(1,N);
iters.luinc01 = zeros(1,N);  time.luinc01 = zeros(1,N);
iters.luinc001 = zeros(1,N); time.luinc001 = zeros(1,N);
iters.luinc00 = zeros(1,N);  time.luinc00 = zeros(1,N);


for ctr = 1:N
  n = ndom(ctr);
  dim = n^2;
  disp(['n = ',num2str(n)]);  
  
  h = 1/(n-1); % fictive step size
  ud = zeros(4*n-4,1); % fictive boundary conditions

  % some random rhs 
  f = rand((n-2)*(n-2),1);  
  
  A = PoissonMatrix_Dirichlet(h,n);
  rhs = PoissonRHS_Dirichlet(h,n,f,ud);
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % iteration
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % WITHOUT ANY PRECONDITIONING
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % makes no sense, as divergent
  
  %P = speye(dim,dim);
  %[U,iter,res]=Richardson(A,rhs,P,maxit,toler);  
  
  %iters.Richardson(ctr) = iter;
    
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % LUINC
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  tic;
  [Low,Up] = luinc(A,'0');
  t = toc;
  time.luinc0(ctr) = t;
  
  % for more efficiency, put explicit implementation here
  normRHS = norm(rhs,2);

  % initialise
  U = zeros(dim,1);

  % for residual visualisation
  residuals = NaN*zeros(maxit,1);

  Nb = Up\(Low\rhs); % preconditioned rhs
  for iter=1:maxit
   U = U - Up\(Low\(A*U)) + Nb;
 
   res = norm(A*U-rhs,2)/normRHS;
 
   residuals(iter) = res;
 
   if res<=toler
     break;
   end
  end

  
  iters.luinc0(ctr) = iter;
      
  
  % set if 1 to visualise the spectrum of iteration matrix
  if 0
    figure
    % iteration matrix
    % without preconditioning
    M = speye(dim,dim) - A;
    E = eig(full(M));
    subplot(2,1,1)
    plot(real(E),imag(E),'*'); 
    
    % with luinc preconditioner
    M = speye(dim,dim) - Up\(Low\A);
    E = eig(full(M));
    
    subplot(2,1,2)
    plot(real(E),imag(E),'*');
    title('luinc')
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  tic;
  [Low,Up] = luinc(A,1);
  t = toc;
  time.luinc1(ctr) = t;
  
  % for more efficiency, put explicit implementation here
  normRHS = norm(rhs,2);

  % initialise
  U = zeros(dim,1);

  % for residual visualisation
  residuals = NaN*zeros(maxit,1);

  Nb = Up\(Low\rhs); % preconditioned rhs
  for iter=1:maxit
   U = U - Up\(Low\(A*U)) + Nb;
 
   res = norm(A*U-rhs,2)/normRHS;
 
   residuals(iter) = res;
 
   if res<=toler
     break;
   end
  end

  
  iters.luinc1(ctr) = iter;
      
  
  if 0
    figure
    % iteration matrix
    % without preconditioning
    M = speye(dim,dim) - A;
    E = eig(full(M));
    subplot(2,1,1)
    plot(real(E),imag(E),'*'); 
    
    % with luinc preconditioner
    M = speye(dim,dim) - Up\(Low\A);
    E = eig(full(M));
    
    subplot(2,1,2)
    plot(real(E),imag(E),'*');
    title('luinc 1')
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

  tic;  
  [Low,Up] = luinc(A,0.1);
  t = toc;
  time.luinc01(ctr) = t;
  
  % for more efficiency, put explicit implementation here
  normRHS = norm(rhs,2);

  % initialise
  U = zeros(dim,1);

  % for residual visualisation
  residuals = NaN*zeros(maxit,1);

  Nb = Up\(Low\rhs); % preconditioned rhs
  for iter=1:maxit
   U = U - Up\(Low\(A*U)) + Nb;
 
   res = norm(A*U-rhs,2)/normRHS;
 
   residuals(iter) = res;
 
   if res<=toler
     break;
   end
  end

  
  iters.luinc01(ctr) = iter;
      
  
  if 0
    figure
    % iteration matrix
    % without preconditioning
    M = speye(dim,dim) - A;
    E = eig(full(M));
    subplot(2,1,1)
    plot(real(E),imag(E),'*'); 
    
    % with luinc preconditioner
    M = speye(dim,dim) - Up\(Low\A);
    E = eig(full(M));
    
    subplot(2,1,2)
    plot(real(E),imag(E),'*');
    title('luinc 0.1')
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
  
  tic;
  [Low,Up] = luinc(A,0.01);
  t = toc;
  time.luinc001(ctr) = t;
  
  % for more efficiency, put explicit implementation here
  normRHS = norm(rhs,2);

  % initialise
  U = zeros(dim,1);

  % for residual visualisation
  residuals = NaN*zeros(maxit,1);

  Nb = Up\(Low\rhs); % preconditioned rhs
  for iter=1:maxit
   U = U - Up\(Low\(A*U)) + Nb;
 
   res = norm(A*U-rhs,2)/normRHS;
 
   residuals(iter) = res;
 
   if res<=toler
     break;
   end
  end

  
  iters.luinc001(ctr) = iter;
      
  
  if 0 
    figure
    % iteration matrix
    % without preconditioning
    M = speye(dim,dim) - A;
    E = eig(full(M));
    subplot(2,1,1)
    plot(real(E),imag(E),'*'); 
    
    % with luinc preconditioner
    M = speye(dim,dim) - Up\(Low\A);
    E = eig(full(M));
    
    subplot(2,1,2)
    plot(real(E),imag(E),'*');
    title('luinc 0.01');
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
  
  tic; 
  [Low,Up] = luinc(A,0.0);
  t = toc;
  time.luinc00(ctr) = t;
  
  % for more efficiency, put explicit implementation here
  normRHS = norm(rhs,2);

  % initialise
  U = zeros(dim,1);

  % for residual visualisation
  residuals = NaN*zeros(maxit,1);

  Nb = Up\(Low\rhs); % preconditioned rhs
  for iter=1:maxit
   U = U - Up\(Low\(A*U)) + Nb;
 
   res = norm(A*U-rhs,2)/normRHS;
 
   residuals(iter) = res;
 
   if res<=toler
     break;
   end
  end

  
  iters.luinc00(ctr) = iter;
      
  
  if 0 
    figure
    % iteration matrix
    % without preconditioning
    M = speye(dim,dim) - A;
    E = eig(full(M));
    subplot(2,1,1)
    plot(real(E),imag(E),'*'); 
    
    % with luinc preconditioner
    M = speye(dim,dim) - Up\(Low\A);
    E = eig(full(M));
    
    subplot(2,1,2)
    plot(real(E),imag(E),'*');
    title('luinc 0')
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
 
  
end


figure
loglog(ndom,iters.luinc0,'b*',...
       ndom,iters.luinc1,'g',...
       ndom,iters.luinc01,'c',...
       ndom,iters.luinc001,'y',...
       ndom,iters.luinc00,'r*')
title('iteration count');
legend('luinc','1','0.1','0.01','0')

figure
loglog(ndom,time.luinc0,'b*',...
       ndom,time.luinc1,'g',...
       ndom,time.luinc01,'c',...
       ndom,time.luinc001,'y',...
       ndom,time.luinc00,'r*')
legend('luinc','1','0.1','0.01','0')       
title('time');




